package csv.to.wf;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceFactoryRegistryImpl;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.m2m.atl.emftvm.EmftvmFactory;
import org.eclipse.m2m.atl.emftvm.ExecEnv;
import org.eclipse.m2m.atl.emftvm.Metamodel;
import org.eclipse.m2m.atl.emftvm.Model;
import org.eclipse.m2m.atl.emftvm.impl.resource.EMFTVMResourceFactoryImpl;
import org.eclipse.m2m.atl.emftvm.util.DefaultModuleResolver;
import org.eclipse.m2m.atl.emftvm.util.ModuleResolver;
import org.eclipse.m2m.atl.emftvm.util.TimingData;

import WorkflowLog.WorkflowLogPackage;
import WorkflowLog.util.WorkflowLogResourceFactoryImpl;
import WorkflowLog.util.WorkflowLogSerializer;
import csv.CSVFile;
import csv.CsvFactory;
import csv.CsvPackage;
import csv.Line;

public class Value2EventExporter {
	
	

    public void start(String pathToCSVFile) {

        String csvFile = pathToCSVFile;
        String cvsSplitBy = ";";

        generateCSVModel(csvFile, cvsSplitBy);
        runTrafo();
		extractXML();
 
    }

	private void generateCSVModel(String csvFile, String cvsSplitBy) {
		
		CsvFactory factory = CsvPackage.eINSTANCE.getCsvFactory();
        CSVFile file = factory.createCSVFile();
        
		String line = "";
        try (BufferedReader br = new BufferedReader(new FileReader(csvFile))) {

            while ((line = br.readLine()) != null) {
                String[] entry = line.split(cvsSplitBy);
                Line lineEntry = factory.createLine();
                lineEntry.setProcessInstance(Integer.parseInt(entry[0]));
                lineEntry.setEvent(entry[1]);
                lineEntry.setTimeStamp(entry[2]);
                file.getLines().add(lineEntry);
            }

        } catch (IOException e) {
            e.printStackTrace();
        }

        store(file,  "output/csvFile.xmi");
	}
    
    public static void extractXML(){
    	
		ResourceSet resourceSet = null;

		resourceSet = new ResourceSetImpl();

		ResourceFactoryRegistryImpl.INSTANCE.getExtensionToFactoryMap().put("xml",
				new WorkflowLogResourceFactoryImpl());

		ResourceFactoryRegistryImpl.INSTANCE.getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());

		WorkflowLogSerializer serializer = new WorkflowLogSerializer();

		URI modelURI = URI.createFileURI(new File("output/out.xmi").getAbsolutePath());
		try {
			serializer.writeXML(resourceSet.getResource(modelURI, true), "output/out.xml");
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    public static void runTrafo(){
        registerNamespaces();
    	ExecEnv env = EmftvmFactory.eINSTANCE.createExecEnv();
    	ResourceSet rs = new ResourceSetImpl();

    	// Load metamodels
    	Metamodel metaModel = EmftvmFactory.eINSTANCE.createMetamodel();
    	metaModel.setResource(rs.getResource(URI.createURI("http://www.eclipse.org/m2m/atl/2011/EMFTVM"), true));
    	env.registerMetaModel("METAMODEL", metaModel);
    	
		Metamodel IM = EmftvmFactory.eINSTANCE.createMetamodel();
		IM.setResource(rs.getResource(URI.createFileURI("model\\ImportFormat.ecore"), true));
		env.registerMetaModel("State", IM);
		registerPackages(rs, rs.getResource(URI.createFileURI("model\\ImportFormat.ecore"), true));
		
		Metamodel OM = EmftvmFactory.eINSTANCE.createMetamodel();
		OM.setResource(rs.getResource(URI.createFileURI("model\\WorkflowLog.ecore"), true));
		env.registerMetaModel("wf", OM);
		registerPackages(rs, rs.getResource(URI.createFileURI("model\\WorkflowLog.ecore"), true));

    	// Load models
    	Model inModel = EmftvmFactory.eINSTANCE.createModel();
    	inModel.setResource(rs.getResource(URI.createURI("output/csvFile.xmi", true), true));
    	env.registerInputModel("IN", inModel);

    	Model outModel = EmftvmFactory.eINSTANCE.createModel();
    	outModel.setResource(rs.createResource(URI.createFileURI("output/out.xmi")));
    	env.registerOutputModel("OUT", outModel);

    	// Load and run module
    	ModuleResolver mr = new DefaultModuleResolver("./trafo/", new ResourceSetImpl());
    	TimingData td = new TimingData();
    	env.loadModule(mr, "values2events");
    	td.finishLoading();
    	env.run(td);
    	td.finish();

    	try {
			outModel.getResource().save(Collections.emptyMap());
		} catch (IOException e) {
			e.printStackTrace();
		}
 
    }
    
	private static void registerNamespaces() {
		EPackage.Registry.INSTANCE.put(WorkflowLogPackage.eNS_URI, WorkflowLogPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(CsvPackage.eNS_URI, CsvPackage.eINSTANCE);
		EPackage.Registry.INSTANCE.put(EcorePackage.eNS_URI, EcorePackage.eINSTANCE);
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("*", new XMIResourceFactoryImpl());
		
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("ecore", new EcoreResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());
		Resource.Factory.Registry.INSTANCE.getExtensionToFactoryMap().put("emftvm", new EMFTVMResourceFactoryImpl());
	}
    
    
    public static void store(EObject document, String uri) {

    	CsvFactory.eINSTANCE.getEPackage();
        Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
        Map<String, Object> m = reg.getExtensionToFactoryMap();
        m.put("xmi", new XMIResourceFactoryImpl());

        ResourceSet resSet = new ResourceSetImpl();

        Resource resource = resSet.createResource(URI
                        .createURI(uri));
        resource.getContents().add(document);

        try {
                resource.save(Collections.EMPTY_MAP);
        } catch (IOException e) {
                e.printStackTrace();
        }
    }
    
	private static void registerPackages(ResourceSet resourceSet, Resource resource) {
		EList<EObject> eObjects = resource.getContents();
		for(EObject eObject : eObjects){
			if (eObject instanceof EPackage) {
				EPackage p = (EPackage) eObject;
				resourceSet.getPackageRegistry().put(p.getNsURI(), p);
			}
		}
	}

}
